<?php
/**
*
* @package install
* @version $Id: install_install.php 126 2008-07-10 17:52:38Z harmlessgoat22 $
* @copyright (c) 2005 phpBB Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
*/
if (!defined('IN_INSTALL'))
{
	// Someone has tried to access the file direct. This is not a good idea, so exit
	exit;
}

if (!empty($setmodules))
{
	// If phpBB is already installed we do not include this module
	if (isset($config['alphaportal_enabled']))
	{
		return;
	}

	$module[] = array(
		'module_type'		=> 'install',
		'module_title'		=> 'INSTALL',
		'module_filename'	=> substr(basename(__FILE__), 0, -strlen($phpEx)-1),
		'module_order'		=> 10,
		'module_subs'		=> '',
		'module_stages'		=> array('INTRO', 'CREATE_TABLE', 'FINAL'),
		'module_reqs'		=> ''
	);
}

/**
* Installation
* @package install
*/
class install_install extends module
{
	function install_install(&$p_master)
	{
		$this->p_master = &$p_master;
	}

	function main($mode, $sub)
	{
		global $lang, $template, $language, $phpbb_root_path, $cache, $version;

		switch ($sub)
		{
			case 'intro':
				$this->page_title = $lang['SUB_INTRO'];

				$template->assign_vars(array(
					'TITLE'			=> $lang['INSTALL_INTRO'],
					'BODY'			=> $lang['INSTALL_INTRO_BODY'],
					'L_SUBMIT'		=> $lang['NEXT_STEP'],
					'S_LANG_SELECT'	=> '<select id="language" name="language">' . $this->p_master->inst_language_select($language) . '</select>',
					'U_ACTION'		=> $this->p_master->module_url . "?mode=$mode&amp;sub=create_table&amp;language=$language",
				));

			break;

			case 'create_table':
				$this->add_tables($mode, $sub);
				$this->populate_tables();
			break;

			case 'final':
				$this->add_permissions();
				$this->set_configs();
				$this->add_modules_alpha();
				$this->add_modules_admin();
				$cache->purge();
				$this->page_title = $lang['STAGE_FINAL'];

				$template->assign_vars(array(
					'TITLE'			=> $lang['INSTALL_CONGRATS'],
					'BODY'			=> sprintf($lang['INSTALL_CONGRATS_EXPLAIN'], $version),
					'S_LANG_SELECT'	=> '<select id="language" name="language">' . $this->p_master->inst_language_select($language) . '</select>',
				));
			break;
		}

		$this->tpl_name = 'install_install';
	}

	/**
	* Adds tables
	*/
	function add_tables($mode, $sub)
	{
		global $db, $lang, $template, $phpbb_root_path, $phpEx;
		
		$this->create_tables($this->schema_data);

		$this->page_title = $lang['STAGE_CREATE_TABLE'];

		$submit = $lang['NEXT_STEP'];

		$url = $this->p_master->module_url . "?mode=$mode&amp;sub=final";

		$template->assign_vars(array(
			'BODY'		=> $lang['STAGE_CREATE_TABLE_EXPLAIN'],
			'L_SUBMIT'	=> $submit,
			'U_ACTION'	=> $url,
		));
	}

	/**
	* Adds new permissions
	*/
	function add_permissions()
	{
		global $phpbb_root_path, $phpEx;
		// Setup $auth_admin class so we can add permission options
		include($phpbb_root_path . 'includes/acp/auth.' . $phpEx);
		$auth_admin = new auth_admin();
		
		// Add permissions
		$auth_admin->acl_add_option(array(
			'global'   => array(
				'a_manage_alpha_modules',
				'a_manage_alpha_settings',
				'a_manage_alpha_auth',
				'm_alpha_blog_comments_edit',
				'm_alpha_blog_comments_delete',
				'm_alpha_blog_edit',
				'm_alpha_blog_delete',
				'm_alpha_blog_labels_edit',
				'm_alpha_blog_labels_delete',
				'u_alpha_blog_post',
				'u_alpha_blog_read',
				'u_alpha_blog_delete',
				'u_alpha_blog_edit',
				'u_alpha_blog_bbcode',
				'u_alpha_blog_smilies',
				'u_alpha_blog_vote',
				'u_alpha_blog_comments_post',
				'u_alpha_blog_comments_edit',
				'u_alpha_blog_comments_delete',
				'u_alpha_blog_comments_delete_own',
				'u_alpha_blog_labels',
				'u_alpha_blog_labels_delete')
		));
	}

	/**
	* Adds new permissions
	*/
	function set_configs()
	{
		set_config('allow_blog_attach', 1);
		set_module_config(15, 'instance', 1);
		set_module_config(15, 'signature', 0);
		set_module_config(15, 'user_info', 1);
		set_module_config(15, 'info_order', 0);
		set_config('alphaportal_enabled', 1, true);
		set_config('alphaportal_default_template', 1, true);
		set_config('alphaportal_version', '2.0.0', true);
	}

	/**
	* Creates tables
	*/
	function create_tables($schema_data)
	{
		global $db, $dbms, $db_tools;
		$sql = '';

		foreach ($schema_data as $table_name => $table_data)
		{
			// Create Table statement
			$generator = $textimage = false;
			$line = '';

			switch ($dbms)
			{
				case 'mysql_40':
				case 'mysql_41':
				case 'firebird':
				case 'oracle':
				case 'sqlite':
				case 'postgres':
					$line = "CREATE TABLE {$table_name} (\n";
				break;
	
				case 'mssql':
					$line = "CREATE TABLE [{$table_name}] (\n";
				break;
			}
	
			// Table specific so we don't get overlap
			$modded_array = array();
	
			// Write columns one by one...
			foreach ($table_data['COLUMNS'] as $column_name => $column_data)
			{
				// Get type
				if (strpos($column_data[0], ':') !== false)
				{
					list($orig_column_type, $column_length) = explode(':', $column_data[0]);
					if (!is_array($db_tools->dbms_type_map[$dbms][$orig_column_type . ':']))
					{
						$column_type = sprintf($db_tools->dbms_type_map[$dbms][$orig_column_type . ':'], $column_length);
					}
					else
					{
						if (isset($db_tools->dbms_type_map[$dbms][$orig_column_type . ':']['rule']))
						{
							switch ($db_tools->dbms_type_map[$dbms][$orig_column_type . ':']['rule'][0])
							{
								case 'div':
									$column_length /= $db_tools->dbms_type_map[$dbms][$orig_column_type . ':']['rule'][1];
									$column_length = ceil($column_length);
									$column_type = sprintf($db_tools->dbms_type_map[$dbms][$orig_column_type . ':'][0], $column_length);
								break;
							}
						}
	
						if (isset($db_tools->dbms_type_map[$dbms][$orig_column_type . ':']['limit']))
						{
							switch ($db_tools->dbms_type_map[$dbms][$orig_column_type . ':']['limit'][0])
							{
								case 'mult':
									$column_length *= $db_tools->dbms_type_map[$dbms][$orig_column_type . ':']['limit'][1];
									if ($column_length > $db_tools->dbms_type_map[$dbms][$orig_column_type . ':']['limit'][2])
									{
										$column_type = $db_tools->dbms_type_map[$dbms][$orig_column_type . ':']['limit'][3];
										$modded_array[$column_name] = $column_type;
									}
									else
									{
										$column_type = sprintf($db_tools->dbms_type_map[$dbms][$orig_column_type . ':'][0], $column_length);
									}
								break;
							}
						}
					}
					$orig_column_type .= ':';
				}
				else
				{
					$orig_column_type = $column_data[0];
					$column_type = $db_tools->dbms_type_map[$dbms][$column_data[0]];
					if ($column_type == 'text' || $column_type == 'blob')
					{
						$modded_array[$column_name] = $column_type;
					}
				}
	
				// Adjust default value if db-dependant specified
				if (is_array($column_data[1]))
				{
					$column_data[1] = (isset($column_data[1][$dbms])) ? $column_data[1][$dbms] : $column_data[1]['default'];
				}
	
				switch ($dbms)
				{
					case 'mysql_40':
					case 'mysql_41':
						$line .= "\t{$column_name} {$column_type} ";
	
						// For hexadecimal values do not use single quotes
						if (!is_null($column_data[1]) && substr($column_type, -4) !== 'text' && substr($column_type, -4) !== 'blob')
						{
							$line .= (strpos($column_data[1], '0x') === 0) ? "DEFAULT {$column_data[1]} " : "DEFAULT '{$column_data[1]}' ";
						}
						$line .= 'NOT NULL';
	
						if (isset($column_data[2]))
						{
							if ($column_data[2] == 'auto_increment')
							{
								$line .= ' auto_increment';
							}
							else if ($dbms === 'mysql_41' && $column_data[2] == 'true_sort')
							{
								$line .= ' COLLATE utf8_unicode_ci';
							}
						}
		
						$line .= ",\n";
					break;
	
					case 'sqlite':
						if (isset($column_data[2]) && $column_data[2] == 'auto_increment')
						{
							$line .= "\t{$column_name} INTEGER PRIMARY KEY ";
							$generator = $column_name;
						}
						else
						{
							$line .= "\t{$column_name} {$column_type} ";
						}
	
						$line .= 'NOT NULL ';
						$line .= (!is_null($column_data[1])) ? "DEFAULT '{$column_data[1]}'" : '';
						$line .= ",\n";
					break;
	
					case 'firebird':
						$line .= "\t{$column_name} {$column_type} ";
	
						if (!is_null($column_data[1]))
						{
							$line .= 'DEFAULT ' . ((is_numeric($column_data[1])) ? $column_data[1] : "'{$column_data[1]}'") . ' ';
						}
	
						$line .= 'NOT NULL';
	
						// This is a UNICODE column and thus should be given it's fair share
						if (preg_match('/^X?STEXT_UNI|VCHAR_(CI|UNI:?)/', $column_data[0]))
						{
							$line .= ' COLLATE UNICODE';
						}
	
						$line .= ",\n";
	
						if (isset($column_data[2]) && $column_data[2] == 'auto_increment')
						{
							$generator = $column_name;
						}
					break;
	
					case 'mssql':
						if ($column_type == '[text]')
						{
							$textimage = true;
						}
	
						$line .= "\t[{$column_name}] {$column_type} ";
	
						if (!is_null($column_data[1]))
						{
							// For hexadecimal values do not use single quotes
							if (strpos($column_data[1], '0x') === 0)
							{
								$line .= 'DEFAULT (' . $column_data[1] . ') ';
							}
							else
							{
								$line .= 'DEFAULT (' . ((is_numeric($column_data[1])) ? $column_data[1] : "'{$column_data[1]}'") . ') ';
							}
						}
	
						if (isset($column_data[2]) && $column_data[2] == 'auto_increment')
						{
							$line .= 'IDENTITY (1, 1) ';
						}
	
						$line .= 'NOT NULL';
						$line .= " ,\n";
					break;
	
					case 'oracle':
						$line .= "\t{$column_name} {$column_type} ";
						$line .= (!is_null($column_data[1])) ? "DEFAULT '{$column_data[1]}' " : '';
	
						// In Oracle empty strings ('') are treated as NULL.
						// Therefore in oracle we allow NULL's for all DEFAULT '' entries
						$line .= ($column_data[1] === '') ? ",\n" : "NOT NULL,\n";
	
						if (isset($column_data[2]) && $column_data[2] == 'auto_increment')
						{
							$generator = $column_name;
						}
					break;
	
					case 'postgres':
						$line .= "\t{$column_name} {$column_type} ";
		
						if (isset($column_data[2]) && $column_data[2] == 'auto_increment')
						{
							$line .= "DEFAULT nextval('{$table_name}_seq'),\n";
		
							// Make sure the sequence will be created before creating the table
							$line = "CREATE SEQUENCE {$table_name}_seq;\n\n" . $line;
						}
						else
						{
							$line .= (!is_null($column_data[1])) ? "DEFAULT '{$column_data[1]}' " : '';
							$line .= "NOT NULL";
	
							// Unsigned? Then add a CHECK contraint
							if (in_array($orig_column_type, $db_tools->unsigned_types))
							{
								$line .= " CHECK ({$column_name} >= 0)";
							}
	
							$line .= ",\n";
						}
					break;
				}
			}
	
			switch ($dbms)
			{
				case 'firebird':
					// Remove last line delimiter...
					$line = substr($line, 0, -2);
					$line .= "\n);;\n\n";
				break;
	
				case 'mssql':
					$line = substr($line, 0, -2);
					$line .= "\n) ON [PRIMARY]" . (($textimage) ? ' TEXTIMAGE_ON [PRIMARY]' : '') . "\n";
					$line .= "GO\n\n";
				break;
			}
	
			// Write primary key
			if (isset($table_data['PRIMARY_KEY']))
			{
				if (!is_array($table_data['PRIMARY_KEY']))
				{
					$table_data['PRIMARY_KEY'] = array($table_data['PRIMARY_KEY']);
				}
	
				switch ($dbms)
				{
					case 'mysql_40':
					case 'mysql_41':
					case 'postgres':
						$line .= "\tPRIMARY KEY (" . implode(', ', $table_data['PRIMARY_KEY']) . "),\n";
					break;
	
					case 'firebird':
						$line .= "ALTER TABLE {$table_name} ADD PRIMARY KEY (" . implode(', ', $table_data['PRIMARY_KEY']) . ");;\n\n";
					break;
	
					case 'sqlite':
						if ($generator === false || !in_array($generator, $table_data['PRIMARY_KEY']))
						{
							$line .= "\tPRIMARY KEY (" . implode(', ', $table_data['PRIMARY_KEY']) . "),\n";
						}
					break;
	
					case 'mssql':
						$line .= "ALTER TABLE [{$table_name}] WITH NOCHECK ADD \n";
						$line .= "\tCONSTRAINT [PK_{$table_name}] PRIMARY KEY  CLUSTERED \n";
						$line .= "\t(\n";
						$line .= "\t\t[" . implode("],\n\t\t[", $table_data['PRIMARY_KEY']) . "]\n";
						$line .= "\t)  ON [PRIMARY] \n";
						$line .= "GO\n\n";
					break;
	
					case 'oracle':
						$line .= "\tCONSTRAINT pk_{$table_name} PRIMARY KEY (" . implode(', ', $table_data['PRIMARY_KEY']) . "),\n";
					break;
				}
			}
	
			switch ($dbms)
			{
				case 'oracle':
					// UNIQUE contrains to be added?
					if (isset($table_data['KEYS']))
					{
						foreach ($table_data['KEYS'] as $key_name => $key_data)
						{
							if (!is_array($key_data[1]))
							{
								$key_data[1] = array($key_data[1]);
							}
	
							if ($key_data[0] == 'UNIQUE')
							{
								$line .= "\tCONSTRAINT u_phpbb_{$key_name} UNIQUE (" . implode(', ', $key_data[1]) . "),\n";
							}
						}
					}
	
					// Remove last line delimiter...
					$line = substr($line, 0, -2);
					$line .= "\n)\n/\n\n";
				break;
	
				case 'postgres':
					// Remove last line delimiter...
					$line = substr($line, 0, -2);
					$line .= "\n);\n\n";
				break;
	
				case 'sqlite':
					// Remove last line delimiter...
					$line = substr($line, 0, -2);
					$line .= "\n);\n\n";
				break;
			}
	
			// Write Keys
			if (isset($table_data['KEYS']))
			{
				foreach ($table_data['KEYS'] as $key_name => $key_data)
				{
					if (!is_array($key_data[1]))
					{
						$key_data[1] = array($key_data[1]);
					}
	
					switch ($dbms)
					{
						case 'mysql_40':
						case 'mysql_41':
							$line .= ($key_data[0] == 'INDEX') ? "\tKEY" : '';
							$line .= ($key_data[0] == 'UNIQUE') ? "\tUNIQUE" : '';
							foreach ($key_data[1] as $key => $col_name)
							{
								if (isset($modded_array[$col_name]))
								{
									switch ($modded_array[$col_name])
									{
										case 'text':
										case 'blob':
											$key_data[1][$key] = $col_name . '(255)';
										break;
									}
								}
							}
							$line .= ' ' . $key_name . ' (' . implode(', ', $key_data[1]) . "),\n";
						break;
	
						case 'firebird':
							$line .= ($key_data[0] == 'INDEX') ? 'CREATE INDEX' : '';
							$line .= ($key_data[0] == 'UNIQUE') ? 'CREATE UNIQUE INDEX' : '';
	
							$line .= ' ' . $table_name . '_' . $key_name . ' ON ' . $table_name . '(' . implode(', ', $key_data[1]) . ");;\n";
						break;
	
						case 'mssql':
							$line .= ($key_data[0] == 'INDEX') ? 'CREATE  INDEX' : '';
							$line .= ($key_data[0] == 'UNIQUE') ? 'CREATE  UNIQUE  INDEX' : '';
							$line .= " [{$key_name}] ON [{$table_name}]([" . implode('], [', $key_data[1]) . "]) ON [PRIMARY]\n";
							$line .= "GO\n\n";
						break;
	
						case 'oracle':
							if ($key_data[0] == 'UNIQUE')
							{
								continue;
							}
	
							$line .= ($key_data[0] == 'INDEX') ? 'CREATE INDEX' : '';
	
							$line .= " {$table_name}_{$key_name} ON {$table_name} (" . implode(', ', $key_data[1]) . ")\n";
							$line .= "/\n";
						break;
	
						case 'sqlite':
							$line .= ($key_data[0] == 'INDEX') ? 'CREATE INDEX' : '';
							$line .= ($key_data[0] == 'UNIQUE') ? 'CREATE UNIQUE INDEX' : '';
	
							$line .= " {$table_name}_{$key_name} ON {$table_name} (" . implode(', ', $key_data[1]) . ");\n";
						break;
	
						case 'postgres':
							$line .= ($key_data[0] == 'INDEX') ? 'CREATE INDEX' : '';
							$line .= ($key_data[0] == 'UNIQUE') ? 'CREATE UNIQUE INDEX' : '';
	
							$line .= " {$table_name}_{$key_name} ON {$table_name} (" . implode(', ', $key_data[1]) . ");\n";
						break;
					}
				}
			}
	
			switch ($dbms)
			{
				case 'mysql_40':
					// Remove last line delimiter...
					$line = substr($line, 0, -2);
					$line .= "\n);\n\n";
				break;
	
				case 'mysql_41':
					// Remove last line delimiter...
					$line = substr($line, 0, -2);
					$line .= "\n) CHARACTER SET `utf8` COLLATE `utf8_bin`;\n\n";
				break;
	
				// Create Generator
				case 'firebird':
					if ($generator !== false)
					{
						$line .= "\nCREATE GENERATOR {$table_name}_gen;;\n";
						$line .= 'SET GENERATOR ' . $table_name . "_gen TO 0;;\n\n";
	
						$line .= 'CREATE TRIGGER t_' . $table_name . ' FOR ' . $table_name . "\n";
						$line .= "BEFORE INSERT\nAS\nBEGIN\n";
						$line .= "\tNEW.{$generator} = GEN_ID({$table_name}_gen, 1);\nEND;;\n\n";
					}
				break;
	
				case 'oracle':
					if ($generator !== false)
					{
						$line .= "\nCREATE SEQUENCE {$table_name}_seq\n/\n\n";
	
						$line .= "CREATE OR REPLACE TRIGGER t_{$table_name}\n";
						$line .= "BEFORE INSERT ON {$table_name}\n";
						$line .= "FOR EACH ROW WHEN (\n";
						$line .= "\tnew.{$generator} IS NULL OR new.{$generator} = 0\n";
						$line .= ")\nBEGIN\n";
						$line .= "\tSELECT {$table_name}_seq.nextval\n";
						$line .= "\tINTO :new.{$generator}\n";
						$line .= "\tFROM dual;\nEND;\n/\n\n";
					}
				break;
			}
	
			$sql .= "{$line}\n";
			$line = '';
	
			// Write custom function at the end for some db's
			switch ($dbms)
			{
				case 'mssql':
					$line = "\nCOMMIT\nGO\n\n";
				break;
		
				case 'sqlite':
					$line = "\nCOMMIT;";
				break;
			
				case 'postgres':
					$line = "\nCOMMIT;";
				break;
			}
		
			$sql .= $line;
			$db->sql_query($sql);
			$line = '';
			$sql = '';
		}
	}

	/**
	* Table data
	*/
	var $schema_data = array(
		ALPHA_PAGES_TABLE => array(
			'COLUMNS'		=> array(
				'page_id'					=> array('UINT', 0),
				'title'						=> array('VCHAR', ''),
				'content'					=> array('XSTEXT', NULL),
				'bbcode_options'			=> array('TINT:1', 7),
				'bbcode_bitfield'			=> array('VCHAR', 0),
				'bbcode_uid'				=> array('VCHAR:8', 0),
				'in_menu'					=> array('BOOL', 0),
				'style'						=> array('UINT', 0),
			),
			'PRIMARY_KEY'	=> 'page_id',
			'KEYS'			=> array(
				'in_menu'	=> array('INDEX', 'in_menu'),
			),
		),
		ALPHA_MODULES_TABLE => array(
			'COLUMNS'		=> array(
				'module_id'					=> array('UINT', NULL, 'auto_increment'),
				'module_enabled'			=> array('BOOL', 1),
				'module_display'			=> array('BOOL', 1),
				'module_basename'			=> array('VCHAR', ''),
				'module_class'				=> array('VCHAR:10', 'ap'),
				'parent_id'					=> array('UINT', 0),
				'left_id'					=> array('UINT', 0),
				'right_id'					=> array('UINT', 0),
				'module_langname'			=> array('VCHAR', ''),
				'module_mode'				=> array('VCHAR', ''),
				'module_auth'				=> array('VCHAR', ''),
				'predefined'				=> array('BOOL', 0),
			),
			'PRIMARY_KEY'	=> 'module_id',
			'KEYS'			=> array(
				'left_right_id'		=> array('INDEX', array('left_id', 'right_id')),
				'module_enabled'	=> array('INDEX', 'module_enabled'),
				'module_display'	=> array('INDEX', 'module_display'),
				'class_left_id'		=> array('INDEX', array('module_class', 'left_id')),
				'module_langname'	=> array('INDEX', 'module_langname'),
				'module_basename'	=> array('INDEX', 'module_basename'),
				'parent_id'			=> array('INDEX', 'parent_id'),
			),
		),
		ALPHA_PORTAL_TABLE => array(
			'COLUMNS'		=> array(
				'instance_id'				=> array('UINT', NULL, 'auto_increment'),
				'instance_template'			=> array('VCHAR', ''),
				'instance_name'				=> array('VCHAR', ''),
				'instance_desc'				=> array('VCHAR', ''),
			),
			'PRIMARY_KEY'	=> 'instance_id',
		),
		ALPHA_PORTAL_OPTIONS_TABLE => array(
			'COLUMNS'		=> array(
				'option_id'					=> array('UINT', NULL, 'auto_increment'),
				'instance_id'				=> array('UINT', 0),
				'post_id'					=> array('UINT', 0),
				'forum_id'					=> array('UINT', 0),
				'recursive'					=> array('BOOL', 0),
				'post_limit'				=> array('USINT', 0),
				'single_post_topic'			=> array('BOOL', 0),
				'character_limit'			=> array('USINT', 0),
			),
			'PRIMARY_KEY'	=> 'option_id',
		),
		ALPHA_MODULES_CONFIG_TABLE => array(
			'COLUMNS'		=> array(
				'module_id'		=> array('UINT', 0),
				'type'			=> array('VCHAR', ''),
				'value'			=> array('VCHAR', ''),
			),
			'KEYS'			=> array(
				'module_id'		=> array('INDEX', 'module_id'),
			),
		),
		ACL_ALPHA_TABLE => array(
			'COLUMNS'		=> array(
				'group_id'			=> array('UINT', 0),
				'module_id'			=> array('UINT', 0),
				'auth_setting'		=> array('TINT:2', 1),
			),
			'KEYS'			=> array(
				'group_id'		=> array('INDEX', 'group_id'),
			),
		),
		ALPHA_SIDEBARS_TABLE => array(
			'COLUMNS'		=> array(
				'page'			=> array('VCHAR', ''),
				'page_id'		=> array('UINT', 0),
				'title'			=> array('VCHAR', ''),
				'number'		=> array('UINT', 3),
				'user_made'		=> array('BOOL', 0),
			),
			'KEYS'			=> array(
				'page'		=> array('INDEX', 'page'),
			),
		),
		ALPHA_BLOG_POSTS_TABLE => array(
			'COLUMNS'		=> array(
				'blog_id'					=> array('UINT', NULL, 'auto_increment'),
				'poster_id'					=> array('UINT', 0),
				'post_time'					=> array('INT:20', 0),
				'subject'					=> array('VCHAR', ''),
				'content'					=> array('XSTEXT', ''),
				'rating'					=> array('USINT', 0),
				'comment_count'				=> array('UINT', 0),
				'draft'						=> array('BOOL', 0),
				'bbcode_options'			=> array('TINT:1', 7),
				'bbcode_bitfield'			=> array('VCHAR', 0),
				'bbcode_uid'				=> array('VCHAR:8', 0),
			),
			'PRIMARY_KEY'	=> 'blog_id',
			'KEYS'			=> array(
				'poster_id'	=> array('INDEX', 'poster_id'),
			),
		),
		ALPHA_BLOG_COMMENTS_TABLE => array(
			'COLUMNS'		=> array(
				'comment_id'				=> array('UINT', NULL, 'auto_increment'),
				'poster_id'					=> array('UINT', 0),
				'blog_id'					=> array('UINT', 0),
				'post_time'					=> array('INT:20', 0),
				'comment'					=> array('XSTEXT', NULL),
				'bbcode_options'			=> array('TINT:1', 7),
				'bbcode_bitfield'			=> array('VCHAR', 0),
				'bbcode_uid'				=> array('VCHAR:8', 0),
			),
			'PRIMARY_KEY'	=> 'comment_id',
			'KEYS'			=> array(
				'poster_id'	=> array('INDEX', 'poster_id'),
				'blog_id'	=> array('INDEX', 'blog_id'),
			),
		),
		ALPHA_BLOG_POST_LABELS_TABLE => array(
			'COLUMNS'		=> array(
				'blog_id'			=> array('UINT', 0),
				'label_id'			=> array('UINT', 0),
			),
			'KEYS'			=> array(
				'blog_id'	=> array('INDEX', 'blog_id'),
				'label_id'	=> array('INDEX', 'label_id'),
			),
		),

		ALPHA_BLOG_LABELS_TABLE => array(
			'COLUMNS'		=> array(
				'label_id'		=> array('UINT', NULL, 'auto_increment'),
				'user_id'		=> array('UINT', 0),
				'name'			=> array('VCHAR', ''),
			),
			'PRIMARY_KEY'	=> 'label_id',
			'KEYS'			=> array(
				'user_id'	=> array('INDEX', 'user_id'),
			),
		),

		ALPHA_BLOG_RATINGS_TABLE => array(
			'COLUMNS'		=> array(
				'blog_id'			=> array('UINT', 0),
				'user_id'			=> array('UINT', 0),
				'rating'			=> array('TINT:1', 0),
			),
			'KEYS'			=> array(
				'blog_id'	=> array('INDEX', 'blog_id'),
				'user_id'	=> array('INDEX', 'user_id'),
			),
		),
	);

	/**
	* Add modules for administrative use
	*/
	function add_modules_admin()
	{
		global $db;

		$sql = 'SELECT * FROM ' . MODULES_TABLE . " WHERE module_langname = 'ACP_CAT_DOT_MODS'";
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!sizeof($row))
		{
			$sql = 'SELECT * FROM ' . MODULES_TABLE . " WHERE module_class = 'acp' ORDER BY right_id DESC";
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);
		}

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> '',
			'module_class'			=> 'acp',
			'parent_id'				=> $row['module_id'],
			'left_id'				=> $row['right_id'],
			'right_id'				=> $row['right_id'] + 9,
			'module_langname'		=> 'ACP_ALPHA',
			'module_mode'			=> '',
			'module_auth'			=> '',
		);

		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
		$module_id = $db->sql_nextid();

		$sql = 'UPDATE ' . MODULES_TABLE . "
		SET left_id = left_id + 10, right_id = right_id + 10
		WHERE left_id >= {$sql_ary['left_id']} AND module_id != $module_id";
		$db->sql_query($sql);
			
		$sql = 'UPDATE ' . MODULES_TABLE . "
		SET right_id = right_id + 10
		WHERE left_id < {$sql_ary['left_id']} AND right_id >= {$sql_ary['left_id']} AND module_id != $module_id";
		$db->sql_query($sql);

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> 'alpha',
			'module_class'			=> 'acp',
			'parent_id'				=> $module_id,
			'left_id'				=> $row['right_id'] + 1,
			'right_id'				=> $row['right_id'] + 2,
			'module_langname'		=> 'ACP_ALPHA_SETTINGS',
			'module_mode'			=> 'settings',
			'module_auth'			=> 'acl_a_manage_alpha_settings',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> 'alpha',
			'module_class'			=> 'acp',
			'parent_id'				=> $module_id,
			'left_id'				=> $row['right_id'] + 3,
			'right_id'				=> $row['right_id'] + 4,
			'module_langname'		=> 'ACP_ALPHA_AUTH',
			'module_mode'			=> 'auth',
			'module_auth'			=> 'acl_a_manage_alpha_auth',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> 'alpha',
			'module_class'			=> 'acp',
			'parent_id'				=> $module_id,
			'left_id'				=> $row['right_id'] + 5,
			'right_id'				=> $row['right_id'] + 6,
			'module_langname'		=> 'ACP_ALPHA_MODULE_MANAGEMENT',
			'module_mode'			=> 'modules',
			'module_auth'			=> 'acl_a_manage_alpha_modules',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> 'alpha',
			'module_class'			=> 'acp',
			'parent_id'				=> $module_id,
			'left_id'				=> $row['right_id'] + 7,
			'right_id'				=> $row['right_id'] + 8,
			'module_langname'		=> 'ACP_ALPHA_PORTAL',
			'module_mode'			=> 'portal',
			'module_auth'			=> 'acl_a_manage_alpha_modules',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql = 'SELECT * FROM ' . MODULES_TABLE . " WHERE module_langname = 'UCP_ZEBRA'";
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!sizeof($row))
		{
			$sql = 'SELECT * FROM ' . MODULES_TABLE . " WHERE module_class = 'ucp' ORDER BY right_id DESC";
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);
		}

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> '',
			'module_class'			=> 'ucp',
			'parent_id'				=> 0,
			'left_id'				=> $row['right_id'] + 1,
			'right_id'				=> $row['right_id'] + 10,
			'module_langname'		=> 'UCP_ALPHA_BLOG',
			'module_mode'			=> '',
			'module_auth'			=> '',
		);

		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
		$module_id = $db->sql_nextid();

		$sql = 'UPDATE ' . MODULES_TABLE . "
		SET left_id = left_id + 10, right_id = right_id + 10
		WHERE left_id >= {$sql_ary['left_id']} AND module_id != $module_id";
		$db->sql_query($sql);
			
		$sql = 'UPDATE ' . MODULES_TABLE . "
		SET right_id = right_id + 10
		WHERE left_id < {$sql_ary['left_id']} AND right_id >= {$sql_ary['left_id']} AND module_id != $module_id";
		$db->sql_query($sql);

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> 'alpha_blog',
			'module_class'			=> 'ucp',
			'parent_id'				=> $module_id,
			'left_id'				=> $row['right_id'] + 2,
			'right_id'				=> $row['right_id'] + 3,
			'module_langname'		=> 'UCP_ALPHA_BLOG_POST',
			'module_mode'			=> 'posting',
			'module_auth'			=> 'acl_u_alpha_blog_post',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 0,
			'module_basename'		=> 'alpha_blog',
			'module_class'			=> 'ucp',
			'parent_id'				=> $module_id,
			'left_id'				=> $row['right_id'] + 4,
			'right_id'				=> $row['right_id'] + 5,
			'module_langname'		=> 'UCP_ALPHA_BLOG_VIEW_POST',
			'module_mode'			=> 'view_post',
			'module_auth'			=> 'acl_u_alpha_blog_post',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> 'alpha_blog',
			'module_class'			=> 'ucp',
			'parent_id'				=> $module_id,
			'left_id'				=> $row['right_id'] + 6,
			'right_id'				=> $row['right_id'] + 7,
			'module_langname'		=> 'UCP_ALPHA_BLOG_VIEW_ALL',
			'module_mode'			=> 'view_all',
			'module_auth'			=> 'acl_u_alpha_blog_post',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_basename'		=> 'alpha_blog',
			'module_class'			=> 'ucp',
			'parent_id'				=> $module_id,
			'left_id'				=> $row['right_id'] + 8,
			'right_id'				=> $row['right_id'] + 9,
			'module_langname'		=> 'UCP_ALPHA_BLOG_LABELS',
			'module_mode'			=> 'labels',
			'module_auth'			=> 'acl_u_alpha_blog_labels',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
	}

	/**
	* Add alphaPORTAL modules
	*/
	function add_modules_alpha()
	{
		global $db;
		$sql_ary = array(
			'module_id'				=> 1,
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_class'			=> 'ap',
			'parent_id'				=> 0,
			'left_id'				=> 1,
			'right_id'				=> 4,
			'module_langname'		=> 'AP_LEFT',
			'predefined'			=> 1,
		);
		$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

				$sql_ary = array(
					'module_id'				=> 10,
					'module_enabled'		=> 1,
					'module_display'		=> 1,
					'module_basename'		=> 'menu',
					'module_class'			=> 'ap',
					'parent_id'				=> 1,
					'left_id'				=> 2,
					'right_id'				=> 3,
					'module_langname'		=> 'AP_MENU',
					'module_mode'			=> 'menu',
				);
				$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);

		$sql_ary = array(
			'module_id'				=> 2,
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_class'			=> 'ap',
			'parent_id'				=> 0,
			'left_id'				=> 5,
			'right_id'				=> 8,
			'module_langname'		=> 'AP_RIGHT',
			'predefined'			=> 1,
		);
		$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

				$sql_ary = array(
					'module_id'				=> 11,
					'module_enabled'		=> 1,
					'module_display'		=> 1,
					'module_basename'		=> 'user_info',
					'module_class'			=> 'ap',
					'parent_id'				=> 2,
					'left_id'				=> 6,
					'right_id'				=> 7,
					'module_langname'		=> 'AP_LOGIN_LOGOUT',
					'module_mode'			=> 'login_logout',
				);
				$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);

		$sql_ary = array(
			'module_id'				=> 3,
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_class'			=> 'ap',
			'parent_id'				=> 0,
			'left_id'				=> 9,
			'right_id'				=> 14,
			'module_langname'		=> 'AP_COMBINED',
			'predefined'			=> 1,
		);
		$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

				$sql_ary = array(
					'module_id'				=> 12,
					'module_enabled'		=> 1,
					'module_display'		=> 1,
					'module_basename'		=> 'menu',
					'module_class'			=> 'ap',
					'parent_id'				=> 3,
					'left_id'				=> 10,
					'right_id'				=> 11,
					'module_langname'		=> 'AP_MENU',
					'module_mode'			=> 'menu',
				);
				$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);

				$sql_ary = array(
					'module_id'				=> 13,
					'module_enabled'		=> 1,
					'module_display'		=> 1,
					'module_basename'		=> 'user_info',
					'module_class'			=> 'ap',
					'parent_id'				=> 3,
					'left_id'				=> 12,
					'right_id'				=> 13,
					'module_langname'		=> 'AP_LOGIN_LOGOUT',
					'module_mode'			=> 'login_logout',
				);
				$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);
	
		$sql_ary = array(
			'module_id'				=> 4,
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_class'			=> 'ap',
			'parent_id'				=> 0,
			'left_id'				=> 15,
			'right_id'				=> 18,
			'module_langname'		=> 'AP_INDEX',
			'predefined'			=> 1,
		);
		$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

				$sql_ary = array(
					'module_id'				=> 14,
					'module_enabled'		=> 1,
					'module_display'		=> 1,
					'module_basename'		=> 'index',
					'module_class'			=> 'ap',
					'parent_id'				=> 4,
					'left_id'				=> 16,
					'right_id'				=> 17,
					'module_langname'		=> 'AP_FORUMLIST',
					'module_mode'			=> 'index',
				);
				$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);

		$sql_ary = array(
			'module_id'				=> 5,
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_class'			=> 'ap',
			'parent_id'				=> 0,
			'left_id'				=> 19,
			'right_id'				=> 22,
			'module_langname'		=> 'AP_PORTAL',
			'predefined'			=> 1,
		);
		$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

				$sql_ary = array(
					'module_id'				=> 15,
					'module_enabled'		=> 1,
					'module_display'		=> 1,
					'module_basename'		=> 'portal',
					'module_class'			=> 'ap',
					'parent_id'				=> 5,
					'left_id'				=> 20,
					'right_id'				=> 21,
					'module_langname'		=> 'AP_PORTAL',
					'module_mode'			=> 'portal',
				);
				$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);

		$sql_ary = array(
			'module_id'				=> 6,
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_class'			=> 'ap',
			'parent_id'				=> 0,
			'left_id'				=> 23,
			'right_id'				=> 24,
			'module_langname'		=> 'AP_PAGES',
			'predefined'			=> 1,
		);
		$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'module_id'				=> 7,
			'module_enabled'		=> 1,
			'module_display'		=> 1,
			'module_class'			=> 'ap',
			'parent_id'				=> 0,
			'left_id'				=> 25,
			'right_id'				=> 32,
			'module_langname'		=> 'AP_BLOG',
			'predefined'			=> 1,
		);
		$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

				$sql_ary = array(
					'module_id'				=> 8,
					'module_enabled'		=> 1,
					'module_display'		=> 1,
					'module_class'			=> 'ap',
					'parent_id'				=> 7,
					'left_id'				=> 26,
					'right_id'				=> 29,
					'module_langname'		=> 'AP_BLOG_SIDEBAR',
					'predefined'			=> 1,
				);
				$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);

						$sql_ary = array(
							'module_id'				=> 16,
							'module_enabled'		=> 1,
							'module_display'		=> 1,
							'module_basename'		=> 'user_info',
							'module_class'			=> 'ap',
							'parent_id'				=> 8,
							'left_id'				=> 27,
							'right_id'				=> 28,
							'module_langname'		=> 'AP_BLOG_USER_INFO',
							'module_mode'			=> 'blog_user_info',
						);
						$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
						$db->sql_query($sql);

				$sql_ary = array(
					'module_id'				=> 9,
					'module_enabled'		=> 1,
					'module_display'		=> 1,
					'module_class'			=> 'ap',
					'parent_id'				=> 7,
					'left_id'				=> 30,
					'right_id'				=> 31,
					'module_langname'		=> 'AP_BLOG_MAIN',
					'predefined'			=> 1,
				);
				$sql = 'INSERT INTO ' . ALPHA_MODULES_TABLE . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);
	}

	/**
	* Populated newly created tables
	*/
	function populate_tables()
	{
		global $db, $db_tools;
		$sql_ary = array(
			'instance_name'		=> 'Portal',
			'instance_desc'		=> 'The instance used for portal.php',
			'instance_template'	=> 'alphaportal/portal.html',
		);

		$sql = 'INSERT INTO ' . ALPHA_PORTAL_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
		$next = $db->sql_nextid();
		$sql_ary = array(
			'instance_id'		=> $next,
			'forum_id'			=> 2,
		);

		$sql = 'INSERT INTO ' . ALPHA_PORTAL_OPTIONS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'index',
			'title'				=> 'SIDE_INDEX',
			'number'			=> 3,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'portal',
			'title'				=> 'SIDE_PORTAL',
			'number'			=> 3,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'pages',
			'title'				=> 'SIDE_PAGES',
			'number'			=> 3,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'blog',
			'title'				=> 'SIDE_BLOG',
			'number'			=> 3,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'viewforum',
			'title'				=> 'SIDE_VIEW_FORUM',
			'number'			=> 2,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'viewtopic',
			'title'				=> 'SIDE_VIEW_TOPIC',
			'number'			=> 2,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'faq',
			'title'				=> 'SIDE_FAQ',
			'number'			=> 3,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'search',
			'title'				=> 'SIDE_SEARCH',
			'number'			=> 3,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'memberlist',
			'title'				=> 'SIDE_MEMBERLIST',
			'number'			=> 2,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
			'page'				=> 'viewonline',
			'title'				=> 'SIDE_VIEW_ONLINE',
			'number'			=> 3,
			'user_made'			=> 0,
		);
		$sql = 'INSERT INTO ' . ALPHA_SIDEBARS_TABLE . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$db_tools->sql_column_add(USERS_TABLE, 'user_blogs', array('UINT', 0));
	}
}

?>